@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package Test::LWP::UserAgent; # git description: v0.027-10-g2b4229c
+package Test::LWP::UserAgent; # git description: v0.031-3-gf621b26
+# vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: A LWP::UserAgent suitable for simulating and testing network calls
 # KEYWORDS: testing useragent networking mock server client
-# vim: set ts=8 sts=4 sw=4 tw=78 et :
 
-our $VERSION = '0.028';
+our $VERSION = '0.032';
 
 use parent 'LWP::UserAgent';
 use Scalar::Util qw(blessed reftype);
@@ -18,7 +18,7 @@ use HTTP::Status qw(:constants status_message);
 use Try::Tiny;
 use Safe::Isa;
 use Carp;
-use namespace::clean;
+use namespace::clean 0.19 -also => [qw(__isa_coderef __is_regexp __isa_response)];
 
 my @response_map;
 my $network_fallback;
@@ -45,7 +45,7 @@ sub new
 
 sub map_response
 {
-    my ($self, $request_description, $response) = @_;
+    my ($self, $request_specification, $response) = @_;
 
     if (not defined $response and blessed $self)
     {
@@ -53,56 +53,53 @@ sub map_response
         my $matched;
         foreach my $mapping (@{$self->{__response_map}})
         {
-            if ($mapping->[0] eq $request_description)
+            if ($mapping->[0] eq $request_specification)
             {
                 $matched = 1;
                 undef $mapping->[1];
             }
         }
 
-        push @{$self->{__response_map}}, [ $request_description, undef ]
+        push @{$self->{__response_map}}, [ $request_specification, undef ]
             if not $matched;
 
         return;
     }
 
-    if (not $response->$_isa('HTTP::Response') and try { $response->can('request') })
+    my ($isa_response, $error_message) = __isa_response($response);
+    if (not $isa_response)
     {
-        my $oldres = $response;
-        $response = sub {
-            $oldres->request($_[0]) };
+        if (try { $response->can('request') })
+        {
+            my $oldres = $response;
+            $response = sub { $oldres->request($_[0]) };
+        }
+        else
+        {
+            carp 'map_response: ', $error_message;
+        }
     }
 
-    carp 'map_response: response is not a coderef or an HTTP::Response, it\'s a ',
-            (blessed($response) || 'non-object')
-        unless __isa_coderef($response) or $response->$_isa('HTTP::Response');
-
     if (blessed $self)
     {
-        push @{$self->{__response_map}}, [ $request_description, $response ];
+        push @{$self->{__response_map}}, [ $request_specification, $response ];
     }
     else
     {
-        push @response_map, [ $request_description, $response ];
+        push @response_map, [ $request_specification, $response ];
     }
     return $self;
 }
 
 sub map_network_response
 {
-    my ($self, $request_description) = @_;
+    my ($self, $request_specification) = @_;
+
+    push (
+        @{ blessed($self) ? $self->{__response_map} : \@response_map },
+        [ $request_specification, $self->_response_send_request ],
+    );
 
-    if (blessed $self)
-    {
-        # we cannot call ::request here, or we end up in an infinite loop
-        push @{$self->{__response_map}},
-            [ $request_description, sub { $self->SUPER::send_request($_[0]) } ];
-    }
-    else
-    {
-        push @response_map,
-            [ $request_description, sub { LWP::UserAgent->new->send_request($_[0]) } ];
-    }
     return $self;
 }
 
@@ -133,13 +130,7 @@ sub register_psgi
     carp 'register_psgi: app is not a coderef, it\'s a ', ref($app)
         unless __isa_coderef($app);
 
-    carp 'register_psgi: did you forget to load HTTP::Message::PSGI?'
-        unless HTTP::Request->can('to_psgi') and HTTP::Response->can('from_psgi');
-
-    return $self->map_response(
-        $domain,
-        sub { HTTP::Response->from_psgi($app->($_[0]->to_psgi)) },
-    );
+    return $self->map_response($domain, $self->_psgi_to_response($app));
 }
 
 sub unregister_psgi
@@ -287,7 +278,8 @@ sub send_request
     if (not $response->$_isa('HTTP::Response'))
     {
         carp 'response from coderef is not a HTTP::Response, it\'s a ',
-            (blessed($response) || 'non-object');
+            (blessed($response) || ( ref($response) ? ('unblessed ' . ref($response)) : 'non-reference' ));
+
         $response = LWP::UserAgent::_new_response($request, HTTP_INTERNAL_SERVER_ERROR, status_message(HTTP_INTERNAL_SERVER_ERROR));
     }
     else
@@ -317,6 +309,27 @@ sub send_request
     return $response;
 }
 
+# turns a PSGI app into a subref returning an HTTP::Response
+sub _psgi_to_response
+{
+    my ($self, $app) = @_;
+
+    carp 'register_psgi: did you forget to load HTTP::Message::PSGI?'
+        unless HTTP::Request->can('to_psgi') and HTTP::Response->can('from_psgi');
+
+    return sub { HTTP::Response->from_psgi($app->($_[0]->to_psgi)) };
+}
+
+# returns a subref that returns an HTTP::Response from a real network request
+sub _response_send_request
+{
+    my $self = shift;
+
+    # we cannot call ::request here, or we end up in an infinite loop
+    return sub { $self->SUPER::send_request($_[0]) } if blessed $self;
+    return sub { LWP::UserAgent->new->send_request($_[0]) };
+}
+
 sub __isa_coderef
 {
     ref $_[0] eq 'CODE'
@@ -326,7 +339,17 @@ sub __isa_coderef
 
 sub __is_regexp
 {
-    re->can('is_regexp') ? re::is_regexp(shift) : ref(shift) eq 'Regexp';
+    re->can('is_regexp') ? re::is_regexp($_[0]) : ref($_[0]) eq 'Regexp';
+}
+
+# returns true if is expected type for all response mappings,
+# or (false, error message);
+sub __isa_response
+{
+    __isa_coderef($_[0]) || $_[0]->$_isa('HTTP::Response')
+        ? (1)
+        : (0, 'response is not a coderef or an HTTP::Response, it\'s a '
+                . (blessed($_[0]) || ( ref($_[0]) ? 'unblessed ' . ref($_[0]) : 'non-reference' )));
 }
 
 1;
@@ -343,7 +366,7 @@ Test::LWP::UserAgent - A LWP::UserAgent suitable for simulating and testing netw
 
 =head1 VERSION
 
-version 0.028
+version 0.032
 
 =head1 SYNOPSIS
 
@@ -383,7 +406,7 @@ the network, allowing an interception of that request and simulating a
 particular response.  This greatly facilitates testing of client networking
 code where the server follows a known protocol.
 
-The synopsis describes a classic case where you want to test how your
+The synopsis describes a typical case where you want to test how your
 application reacts to various responses from the server.  This module will let
 you send back various responses depending on the request, without having to
 set up a real server to test against.  This can be invaluable when you need to
@@ -395,7 +418,7 @@ hook into this module; see the documentation for the individual interface
 methods.
 
 You can use a L<PSGI> app to handle the requests - see F<examples/call_psgi.t>
-in this dist, and also L</register_psgi> below.
+in this distribution, and also L</register_psgi> below.
 
 OR, you can route some or all requests through the network as normal, but
 still gain the hooks provided by this class to test what was sent and
@@ -470,9 +493,9 @@ C<< $useragent->network_fallback(<value?>) >>.
 =back
 
 B<All other methods below may be called on a specific object instance, or as a class method.>
-If called as on a blessed object, the action performed or data returned is
-limited to just that object; if called as a class method, the action or data is
-global.
+If the method invoked on a blessed object, the action performed or data returned is
+limited to just that object; if it is called as a class method, the action or data is
+global and affects all instances (although specific instances may have overrides; see below).
 
 =head2 C<map_response($request_specification, $http_response)>
 
@@ -516,28 +539,49 @@ parameters, headers etc) against the provided object.
 
 =back
 
-The response can be represented either as a literal L<HTTP::Request> object, or
-as a coderef that is run at the time of matching, with the request passed as
-the single argument:
+The response can be represented in multiple ways:
+
+=over
+
+=item *
+
+a literal L<HTTP::Response> object:
 
     HTTP::Response->new(...);
 
-or
+=item *
+
+as a coderef that is run at the time of matching, with the request passed as
+the single argument:
 
     sub {
         my $request = shift;
-        HTTP::Response->new(...);
+        return HTTP::Response->new(...);
     }
 
+=item *
+
+=for stopwords thusly
+
+a blessed object that implements the C<request> method, which will be saved as
+a coderef thusly (this allows you to use your own dispatcher implementation):
+
+    sub {
+        my $request = shift;
+        return $response->request($request);
+    }
+
+=back
+
 Instance mappings take priority over global (class method) mappings - if no
 matches are found from mappings added to the instance, the global mappings are
 then examined. When no matches have been found, a 404 response is returned.
 
 This method returns the C<Test::LWP::UserAgent> object or class.
 
-=head2 C<map_network_response($request_description)>
+=head2 C<map_network_response($request_specification)>
 
-Same as C<map_response> above, only requests that match this description will
+Same as C<map_response> above, only requests that match this specification will
 not use a response that you specify, but instead uses a real L<LWP::UserAgent>
 to dispatch your request to the network.
 
@@ -641,7 +685,7 @@ This is the only method from L<LWP::UserAgent> that has been overridden, which
 processes the L<HTTP::Request>, sends to the network, then creates the
 L<HTTP::Response> object from the reply received. Here, we loop through your
 local and global domain registrations, and local and global mappings (in this
-order) and returns the first match found; otherwise, a simple 404 response is
+order) and returns the B<first match found>; otherwise, a simple 404 response is
 returned (unless C<network_fallback> was specified as a constructor option,
 in which case unmatched requests will be delivered to the network.)
 
@@ -698,14 +742,6 @@ throughout. It provides a method (C<last_http_request_sent>) to access the last
 L<HTTP::Request>, for testing things like the URI and headers that your code
 sent to L<LWP::UserAgent>.
 
-=head1 SUPPORT
-
-=for stopwords irc
-
-Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=Test-LWP-UserAgent>
-(or L<bug-Test-LWP-UserAgent@rt.cpan.org>).
-I am also usually active on irc, as 'ether' at C<irc.perl.org>.
-
 =head1 ACKNOWLEDGEMENTS
 
 =for stopwords AirG Yury Zavarin mst
@@ -752,21 +788,34 @@ L<Plack::Test>, L<Plack::Test::ExternalServer>
 
 =back
 
+=head1 SUPPORT
+
+Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=Test-LWP-UserAgent>
+(or L<bug-Test-LWP-UserAgent@rt.cpan.org|mailto:bug-Test-LWP-UserAgent@rt.cpan.org>).
+
+There is also a mailing list available for users of this distribution, at
+L<http://lists.perl.org/list/perl-qa.html>.
+
+There is also an irc channel available for users of this distribution, at
+L<C<#perl> on C<irc.perl.org>|irc://irc.perl.org/#perl-qa>.
+
+I am also usually active on irc, as 'ether' at C<irc.perl.org>.
+
 =head1 AUTHOR
 
 Karen Etheridge <ether@cpan.org>
 
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2012 by Karen Etheridge.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =head1 CONTRIBUTOR
 
 =for stopwords Tom Hukins
 
 Tom Hukins <tom@eborcom.com>
 
+=head1 COPYRIGHT AND LICENCE
+
+This software is copyright (c) 2012 by Karen Etheridge.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
